# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nannyml',
 'nannyml.cli',
 'nannyml.datasets',
 'nannyml.datasets.data',
 'nannyml.drift',
 'nannyml.drift.model_inputs',
 'nannyml.drift.model_inputs.multivariate',
 'nannyml.drift.model_inputs.multivariate.data_reconstruction',
 'nannyml.drift.model_inputs.univariate',
 'nannyml.drift.model_inputs.univariate.statistical',
 'nannyml.drift.model_outputs',
 'nannyml.drift.model_outputs.univariate',
 'nannyml.drift.model_outputs.univariate.statistical',
 'nannyml.drift.target',
 'nannyml.drift.target.target_distribution',
 'nannyml.io',
 'nannyml.performance_calculation',
 'nannyml.performance_estimation',
 'nannyml.performance_estimation.confidence_based',
 'nannyml.plots']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2<3.1',
 'PyYAML>=6.0,<7.0',
 'art>=5.7,<6.0',
 'category-encoders>=2.3.0,<3.0.0',
 'click>=8.1.3,<9.0.0',
 'gcsfs>=2022.5.0,<2023.0.0',
 'joblib>=1.1.0,<2.0.0',
 'kaleido==0.2.1',
 'matplotlib>=3.5.1,<4.0.0',
 'pandas>=1.3.0,<2.0.0',
 'plotly>=5.6.0,<6.0.0',
 'pyarrow>=8.0.0,<9.0.0',
 'pydantic>=1.9.1,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'rich>=12.5.1,<13.0.0',
 'scikit-learn>=1.0.2,<2.0.0',
 'scipy>=1.7.3,<1.8.0',
 'seaborn>=0.11.2,<0.12.0']

entry_points = \
{'console_scripts': ['nannyml = nannyml.cli.cli:cli',
                     'nml = nannyml.cli.cli:cli']}

setup_kwargs = {
    'name': 'nannyml',
    'version': '0.5.1',
    'description': 'NannyML, Your library for monitoring model performance.',
    'long_description': '<p align="center">\n    <img src="media/thumbnail-4.png">\n</p>\n<p align="center">\n    <a href="https://pypi.org/project/nannyml/">\n        <img src="https://img.shields.io/pypi/v/nannyml.svg">\n    </a>\n    <a href="https://pypi.org/project/nannyml/">\n        <img src="https://img.shields.io/pypi/pyversions/nannyml.svg">\n    </a>\n    <a href="https://github.com/nannyml/nannyml/actions/workflows/dev.yml">\n        <img src="https://github.com/NannyML/nannyml/actions/workflows/dev.yml/badge.svg">\n    </a>\n    <a href=\'https://nannyml.readthedocs.io/en/main/?badge=main\'>\n        <img src=\'https://readthedocs.org/projects/nannyml/badge/?version=main\' alt=\'Documentation Status\' />\n    </a>\n    <img alt="PyPI - License" src="https://img.shields.io/pypi/l/nannyml?color=green">\n    <br>\n    <br>\n    <a href="https://www.producthunt.com/posts/nannyml?utm_source=badge-top-post-badge&utm_medium=badge&utm_souce=badge-nannyml" target="_blank"><img src="https://api.producthunt.com/widgets/embed-image/v1/top-post-badge.svg?post_id=346412&theme=light&period=daily" alt="NannyML - OSS&#0032;Python&#0032;library&#0032;for&#0032;detecting&#0032;silent&#0032;ML&#0032;model&#0032;failure | Product Hunt" style="width: 250px; height: 54px;" width="250" height="54" /></a>\n\n</p>\n\n<p align="center">\n    <strong>\n        <a href="https://nannyml.com/">Website</a>\n        •\n        <a href="https://nannyml.readthedocs.io/en/stable/">Docs</a>\n        •\n        <a href="https://join.slack.com/t/nannymlbeta/shared_invite/zt-16fvpeddz-HAvTsjNEyC9CE6JXbiM7BQ">Community Slack</a>\n    </strong>\n</p>\n\n<p align="center">\n  <img src="media/cbpe_v3.gif" alt="animated">\n</p>\n\n\n# 💡 What is NannyML?\nNannyML is an open-source python library that allows you to **estimate post-deployment model performance** (without access to targets), detect data drift, and intelligently link data drift alerts back to changes in model performance. Built for data scientists, NannyML has an easy-to-use interface, interactive visualizations, is completely model-agnostic and currently supports all tabular classification use cases.\n\nThe core contributors of NannyML have researched and developed a novel algorithm for estimating model performance: [confidence-based performance estimation (CBPE)](https://nannyml.readthedocs.io/en/stable/how_it_works/performance_estimation.html). The nansters also invented a new approach to detect [multivariate data drift](https://nannyml.readthedocs.io/en/stable/how_it_works/data_reconstruction.html) using PCA-based data reconstruction.\n\nIf you like what we are working on, be sure to become a Nanster yourself, join our [community slack](https://join.slack.com/t/nannymlbeta/shared_invite/zt-16fvpeddz-HAvTsjNEyC9CE6JXbiM7BQ) <img src="media/slack.png" width=\'25\'> and support us with a GitHub <img src="media/github.png" width=\'25\'> star ⭐.\n\n# ☔ Why use NannyML?\nNannyML closes the loop with performance monitoring and post deployment data science, empowering data scientist to quickly understand and **automatically detect silent model failure**. By using NannyML, data scientists can finally maintain complete visibility and trust in their deployed machine learning models.\nAllowing you to have the following benefits:\n\n* End sleepless nights caused by not knowing your model performance 😴\n* Analyse data drift and model performance **over time**\n* Discover the **root cause** to why your models are not performing as expected\n* **No alert fatigue!** React only when necessary if model performance is impacted\n* **Painless** setup in any environment\n\n# 🧠 GO DEEP\n| NannyML Resources                                       | Description                                                                            |\n|---------------------------------------------------------|----------------------------------------------------------------------------------------|\n| ☎️ **[NannyML 101]**                                    | New to NannyML? Start here!                                                            |\n| 🔮 **[Performance estimation]**                         | How the magic works.                                                                   |\n| 🌍 **[Real world example]**                             | Take a look at a real-world example of NannyML.                                        |\n| 🔑 **[Key concepts]**                                   | Glossary of key concepts we use.                                                       |\n| 🔬 **[Technical reference]**                            | Monitor the performance of your ML models.                                             |\n| 🔎 **[Blog]**                                           | Thoughts on post-deployment data science from the NannyML team.                        |\n| 📬 **[Newsletter]**                                     | All things post-deployment data science. Subscribe to see the latest papers and blogs. |\n| 💎 **[New in v0.5.1]**                                  | New features, bug fixes.                                                               |\n| 🧑\u200d💻 **[Contribute]**                                  | How to contribute to the NannyML project and codebase.                                 |\n| <img src="media/slack.png" width=\'25\'> **[Join slack]** | Need help with your specific use case? Say hi on slack!                                |\n\n[NannyML 101]: https://nannyml.readthedocs.io/en/stable/\n[Performance Estimation]: https://nannyml.readthedocs.io/en/stable/how_it_works/performance_estimation.html\n[Key Concepts]: https://nannyml.readthedocs.io/en/stable/glossary.html\n[Technical Reference]:https://nannyml.readthedocs.io/en/stable/nannyml/modules.html\n[New in v0.5.1]: https://github.com/NannyML/nannyml/releases/latest/\n[Real World Example]: https://nannyml.readthedocs.io/en/stable/examples/california_housing.html\n[Blog]: https://www.nannyml.com/blog\n[Newsletter]:  https://mailchi.mp/022c62281d13/postdeploymentnewsletter\n[Join Slack]: https://join.slack.com/t/nannymlbeta/shared_invite/zt-16fvpeddz-HAvTsjNEyC9CE6JXbiM7BQ\n[Contribute]: https://github.com/NannyML/nannyml/blob/main/CONTRIBUTING.rst\n\n# 🔱 Features\n\n### 1. Performance estimation and monitoring\n\nWhen the actual outcome of your deployed prediction models is delayed, or even when post-deployment target labels are completely absent, you can use NannyML\'s [CBPE-algorithm](https://nannyml.readthedocs.io/en/stable/how_it_works/performance_estimation.html) to **estimate model performance**. This algorithm requires the predicted probabilities of your machine learning model and leverages probability calibration to estimate any traditional binary classification metric (ROC_AUC, Precision, Recall, F1, etc.). Rather than estimating the performance of future model predictions, CBPE estimates the expected model performance of the predictions made at inference time.\n\n<p><img src="docs/_static/tutorial-perf-est-guide-with-ref-roc_auc.svg"></p>\n\nNannyML can also **track the realised performance** of your machine learning model once targets are available.\n\n### 2. Data drift detection\n\nTo detect **multivariate feature drift** NannyML uses [PCA-based data reconstruction](https://nannyml.readthedocs.io/en/main/how_it_works/data_reconstruction.html). Changes in the resulting reconstruction error are monitored over time and data drift alerts are logged when the reconstruction error in a certain period exceeds a threshold. This threshold is calculated based on the reconstruction error observed in the reference period.\n\n<p><img src="docs/_static/drift-guide-multivariate.svg"></p>\n\nNannyML utilises statistical tests to detect **univariate feature drift**. The Kolmogorov–Smirnov test is used for continuous features and the 2-sample chi-squared test for categorical features. The results of these tests are tracked over time, properly corrected to counteract multiplicity and overlayed on the temporal feature distributions. (It is also possible to visualise the test-statistics over time, to get a notion of the drift magnitude.)\n\n<p><img src="docs/_static/drift-guide-joyplot-distance_from_office.svg"><img src="docs/_static/drift-guide-stacked-salary_range.svg"></p>\n\nNannyML uses the same statistical tests to detected **model output drift**.\n\n<p><img src="docs/_static/drift-guide-predictions-joyplot.svg"></p>\n\n**Target distribution drift** is monitored by calculating the mean occurrence of positive events in combination with the 2-sample chi-squared test. Bear in mind that this operation requires the presence of actuals.\n\n<p><img src="docs/_static/target_distribution_metric.svg"></p>\n\n### 3. Intelligent alerting\n\nBecause NannyML can estimate performance, it is possible to weed out data drift alerts that do not impact expected performance, combatting alert fatigue. Besides linking data drift issues to drops in performance it is also possible to prioritise alerts according to other criteria using NannyML\'s Ranker.\n\n\n# 🚀 Getting started\n\n### Install NannyML\n\nFrom PyPI:\n\n```bash\npip install nannyml\n```\n\n**Here be dragons!** Use the latest development version of NannyML at your own risk:\n\n```bash\npython -m pip install git+https://github.com/NannyML/nannyml\n```\n\n### Quick Start\n\n*The following snippet is based on our [latest release](https://github.com/NannyML/nannyml/releases/latest)*.\n\n```python\nimport pandas as pd\nimport nannyml as nml\nfrom IPython.display import display\n\n# Load synthetic data\nreference, analysis, analysis_target = nml.load_synthetic_binary_classification_dataset()\ndisplay(reference.head())\ndisplay(analysis.head())\n\n# Choose a chunker or set a chunk size\nchunk_size = 5000\n\n# Estimate model performance\n# initialize, specify required data columns, fit estimator and estimate\nestimator = nml.CBPE(\n   y_pred_proba=\'y_pred_proba\',\n   y_pred=\'y_pred\',\n   y_true=\'work_home_actual\',\n   timestamp_column_name=\'timestamp\',\n   metrics=[\'roc_auc\'],\n   chunk_size=chunk_size,\n)\nestimator = estimator.fit(reference)\nestimated_performance = estimator.estimate(analysis)\n\n# Show results\nfigure = estimated_performance.plot(kind=\'performance\', metric=\'roc_auc\', plot_reference=True)\nfigure.show()\n\n# Define feature columns\nfeature_column_names = [\n    col for col in reference.columns if col not in [\n        \'timestamp\', \'y_pred_proba\', \'period\', \'y_pred\', \'work_home_actual\', \'identifier\'\n    ]]\n\n# Let\'s initialize the object that will perform the Univariate Drift calculations\nunivariate_calculator = nml.UnivariateStatisticalDriftCalculator(\n    feature_column_names=feature_column_names,\n    timestamp_column_name=\'timestamp\',\n    chunk_size=chunk_size\n)\nunivariate_calculator = univariate_calculator.fit(reference)\nunivariate_results = univariate_calculator.calculate(analysis)\n# Plot drift results for all model inputs\nfor feature in univariate_calculator.feature_column_names:\n    figure = univariate_results.plot(\n        kind=\'feature_drift\',\n        metric=\'statistic\',\n        feature_column_name=feature,\n        plot_reference=True\n    )\n    figure.show()\n\n# Rank features based on number of alerts\nranker = nml.Ranker.by(\'alert_count\')\nranked_features = ranker.rank(univariate_results, only_drifting = False)\ndisplay(ranked_features)\n\ncalc = nml.StatisticalOutputDriftCalculator(\n    y_pred=\'y_pred\',\n    y_pred_proba=\'y_pred_proba\',\n    timestamp_column_name=\'timestamp\'\n)\ncalc.fit(reference)\nresults = calc.calculate(analysis)\n\nfigure = results.plot(kind=\'prediction_drift\', plot_reference=True)\nfigure.show()\n\n# Let\'s initialize the object that will detect Multivariate Drift\nrcerror_calculator = nml.DataReconstructionDriftCalculator(feature_column_names=feature_column_names, timestamp_column_name=\'timestamp\', chunk_size=chunk_size).fit(reference_data=reference)\n# let\'s see Reconstruction error statistics for all available data\nrcerror_results = rcerror_calculator.calculate(analysis)\nfigure = rcerror_results.plot(kind=\'drift\', plot_reference=True)\nfigure.show()\n```\n\n# 📖 Documentation\n\n* Performance monitoring\n  - [Estimated performance](https://nannyml.readthedocs.io/en/main/tutorials/performance_estimation.html)\n  - [Realized performance](https://nannyml.readthedocs.io/en/main/tutorials/performance_calculation.html)\n* Drift detection\n  * [Multivariate feature drift](https://nannyml.readthedocs.io/en/main/tutorials/detecting_data_drift/multivariate_drift_detection.html)\n  - [Univariate feature drift](https://nannyml.readthedocs.io/en/main/tutorials/detecting_data_drift/univariate_drift_detection.html)\n  - [Model output drift](https://nannyml.readthedocs.io/en/main/tutorials/detecting_data_drift/drift_detection_for_model_outputs.html)\n  - [Target distribution drift](https://nannyml.readthedocs.io/en/main/tutorials/detecting_data_drift/drift_detection_for_model_targets.html)\n\n# 🦸 Contributing and Community\n\nWe want to build NannyML together with the community! The easiest to contribute at the moment is to propose new features or log bugs under [issues](https://github.com/NannyML/nannyml/issues). For more information, have a look at [how to contribute](CONTRIBUTING.rst).\n\n# 🙋 Get help\n\nThe best place to ask for help is in the [community slack](https://join.slack.com/t/nannymlbeta/shared_invite/zt-16fvpeddz-HAvTsjNEyC9CE6JXbiM7BQ). Feel free to join and ask questions or raise issues. Someone will definitely respond to you.\n\n# \U0001f977 Stay updated\n\nIf you want to stay up to date with recent changes to the NannyML library, you can subscribe to our [release notes](https://nannyml.substack.com). For thoughts on post-deployment data science from the NannyML team, feel free to visit our [blog](https://www.nannyml.com/blog). You can also sing up for our [newsletter](https://mailchi.mp/022c62281d13/postdeploymentnewsletter), which brings together the best papers, articles, news, and open-source libraries highlighting the ML challenges after deployment.\n\n# 📍 Roadmap\nCurious what we are working on next? Have a look at our [roadmap](https://bit.ly/nannymlroadmap). If you have any questions or if you would like to see things prioritised in a different way, let us know!\n\n# 📄 License\n\nNannyML is distributed under an Apache License Version 2.0. A complete version can be found [here](LICENSE.MD). All contributions will be distributed under this license.\n',
    'author': 'Niels Nuyttens',
    'author_email': 'niels@nannyml.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nannyml/nannyml',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)
